@@ -11,18 +11,17 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import functools
-import warnings
 
 import flask
 from injector import Injector
 from flask import Config, Request
 from werkzeug.local import Local, LocalManager, LocalProxy
-from injector import Module, Provider, Scope, ScopeDecorator, singleton, InstanceProvider
+from injector import Module, Provider, Scope, ScopeDecorator, singleton
 
 
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.4.0'
-__all__ = ['init_app', 'post_init_app', 'request', 'RequestScope', 'Config', 'Request', 'FlaskInjector', ]
+__version__ = '0.5.0'
+__all__ = ['request', 'RequestScope', 'Config', 'Request', 'FlaskInjector', ]
 
 
 def wrap_fun(fun, injector):
@@ -41,13 +40,42 @@ def wrap_fun(fun, injector):
 
         return wrapper
     elif hasattr(fun, 'view_class'):
-        current_class = fun.view_class
+        cls = fun.view_class
+        name = fun.__name__
 
-        def cls(**kwargs):
-            return injector.create_object(
-                current_class, additional_kwargs=kwargs)
+        closure_contents = (c.cell_contents for c in fun.__closure__)
+        fun_closure = dict(zip(fun.__code__.co_freevars, closure_contents))
+        class_args = fun_closure['class_args']
+        assert not class_args, 'Class args are not supported, use kwargs instead'
+        class_kwargs = fun_closure['class_kwargs']
 
-        fun.view_class = cls
+        # This section is flask.views.View.as_view code modified to make the injection
+        # possible without relying on modifying view function in place
+        # Copyright (c) 2014 by Armin Ronacher and Flask contributors, see Flask
+        # license for details
+
+        def view(*args, **kwargs):
+            self = injector.create_object(cls, additional_kwargs=class_kwargs)
+            return self.dispatch_request(*args, **kwargs)
+
+        if cls.decorators:
+            view.__name__ = name
+            view.__module__ = cls.__module__
+            for decorator in cls.decorators:
+                view = decorator(view)
+
+        # We attach the view class to the view function for two reasons:
+        # first of all it allows us to easily figure out what class-based
+        # view this thing came from, secondly it's also used for instantiating
+        # the view class so you can actually replace it with something else
+        # for testing purposes and debugging.
+        view.view_class = cls
+        view.__name__ = name
+        view.__doc__ = cls.__doc__
+        view.__module__ = cls.__module__
+        view.methods = cls.methods
+
+        fun = view
 
     return fun
 
@@ -83,23 +111,12 @@ class RequestScope(Scope):
         self._local_manager = LocalManager([self._locals])
         self.reset()
 
-    try:
-        from injector import BoundProvider  # noqa
-    except ImportError:
-        def get(self, key, provider):
-            try:
-                return self._locals.scope[key]
-            except KeyError:
-                provider = InstanceProvider(provider.get())
-                self._locals.scope[key] = provider
-                return provider
-    else:
-        def get(self, key, provider):
-            try:
-                return self._locals.scope[key]
-            except KeyError:
-                new_provider = self._locals.scope[key] = CachedProviderWrapper(provider)
-                return new_provider
+    def get(self, key, provider):
+        try:
+            return self._locals.scope[key]
+        except KeyError:
+            new_provider = self._locals.scope[key] = CachedProviderWrapper(provider)
+            return new_provider
 
 
 request = ScopeDecorator(RequestScope)
@@ -122,42 +139,34 @@ class FlaskInjector(object):
         :type modules: Iterable of configuration modules
         :rtype: :class:`injector.Injector`
         """
-        injector = _init_app(app, modules, injector, request_scope_class)
-        _post_init_app(app, injector, request_scope_class)
+        injector = injector or Injector()
+        for module in (
+                [FlaskModule(app=app, request_scope_class=request_scope_class)] +
+                list(modules)):
+            injector.binder.install(module)
+
+        for container in (
+                app.view_functions,
+                app.before_request_funcs,
+                app.after_request_funcs,
+                app.teardown_request_funcs,
+                app.template_context_processors,
+                app.jinja_env.globals,
+        ):
+            process_dict(container, injector)
+
+        process_error_handler_spec(app.error_handler_spec, injector)
+
+        def reset_request_scope(*args, **kwargs):
+            injector.get(request_scope_class).reset()
+
+        app.before_request_funcs.setdefault(None, []).insert(0, reset_request_scope)
+        app.teardown_request(reset_request_scope)
 
         self.injector = injector
         self.app = app
 
 
-def init_app(app, modules=[], injector=None, request_scope_class=RequestScope):
-    '''
-    Initializes Injector for the application.
-
-    :param app: Application to configure
-    :param modules: Configuration for newly created :class:`injector.Injector`
-    :param injector: Injector to initialize app with, if not provided
-        a new instance will be created.
-    :type app: :class:`flask.Flask`
-    :type modules: Iterable of configuration modules
-    :rtype: :class:`injector.Injector`
-    '''
-
-    warnings.warn(
-        "init_app and post_init_app are deprecated in favour of FlaskInjector. "
-        "Please consult README for details.")
-    return _init_app(app, modules, injector, request_scope_class)
-
-
-def _init_app(app, modules, injector, request_scope_class):
-    injector = injector or Injector()
-    for module in (
-            [FlaskModule(app=app, request_scope_class=request_scope_class)] +
-            list(modules)):
-        injector.binder.install(module)
-
-    return injector
-
-
 def process_dict(d, injector):
     for key, value in d.items():
         if isinstance(value, list):
@@ -178,40 +187,6 @@ def process_error_handler_spec(spec, injector):
         process_dict(subspec, injector)
 
 
-def post_init_app(app, injector, request_scope_class=RequestScope):
-    '''
-    Needs to be called after all views, signal handlers, template globals
-    and context processors are registered.
-
-    :type app: :class:`flask.Flask`
-    :type injector: :class:`injector.Injector`
-    '''
-    warnings.warn(
-        "init_app and post_init_app are deprecated in favour of FlaskInjector. "
-        "Please consult README for details.")
-    _post_init_app(app, injector, request_scope_class)
-
-
-def _post_init_app(app, injector, request_scope_class):
-    for container in (
-            app.view_functions,
-            app.before_request_funcs,
-            app.after_request_funcs,
-            app.teardown_request_funcs,
-            app.template_context_processors,
-            app.jinja_env.globals,
-    ):
-        process_dict(container, injector)
-
-    process_error_handler_spec(app.error_handler_spec, injector)
-
-    def reset_request_scope(*args, **kwargs):
-        injector.get(request_scope_class).reset()
-
-    app.before_request_funcs.setdefault(None, []).insert(0, reset_request_scope)
-    app.teardown_request(reset_request_scope)
-
-
 class FlaskModule(Module):
     def __init__(self, app, request_scope_class=RequestScope):
         self.app = app
